/* Agent Tcl
   Bob Gray
   10 January 1996

   tclParse.cc

   This file implements various parsing routines.

   Copyright (c) 1995-1996, Bob Gray, Dartmouth College

   See the file "agent.terms" for information on usage and redistribution
   of this file and for a DISCLAIMER OF ALL WARRANTIES.
*/

#ifndef NO_PRAGMAS
#pragma implementation
#endif

#include "platPorting.h"
#include "genUtility.h"
#include "tcl.h"
#include "tclParse.h"

/* parseShortTimeout

   Purpose: Parse the [-time seconds] portion of a command
*/

int parseShortTimeout (Tcl_Interp *interp, double &seconds, int argc, char **argv, int i)
{
    /* check for the -time option */

  if (strcmp (argv[i], "-time")) {
    Tcl_AppendResult (interp, "unknown option \"", argv[i], "\": should be \"-time\"", (char *) NULL);
    return -1;
  } 

    /* get the number of seconds */

  if (++i == argc) {
    Tcl_AppendResult (interp, "\"-time\" must be followed by number of seconds", (char *) NULL);
    return -1;
  } else if (Tcl_GetDouble (interp, argv[i], &seconds) != TCL_OK) { 
    Tcl_AppendResult (interp, ": seconds must be a real number", (char *) NULL);
    return -1;
  } else if (seconds < 0.0) {
    Tcl_AppendResult (interp, "seconds must be 0 or greater", (char *) NULL);
    return -1;
  }

    /* we got the number of seconds */

  return ++i;
}

/* parseTimeout

   Purpose: Parse the [-blocking | -time seconds | -nonblocking] section of a
	    command
*/

int parseTimeout (Tcl_Interp *interp, double &seconds, int argc, char **argv, int i)
{
   /* check for the -time option */

  if (!strcmp (argv[i], "-blocking")) {
    seconds = INFINITY;
  } else if (!strcmp (argv[i], "-nonblocking")) {
    seconds = 0.0; 
  } else if (!strcmp (argv[i], "-time")) {

      /* get the number of seconds */

    if (++i == argc) {
      Tcl_SetResult (interp, "-time must be followed by the number of seconds", TCL_STATIC);
      return -1;
    } else if (!strcmp(argv[i], "forever")) {
      seconds = INFINITY;
    } else if (Tcl_GetDouble (interp, argv[i], &seconds) != TCL_OK) { 
      Tcl_SetResult (interp, "-time must be followed by the number of seconds", TCL_STATIC);
      return -1;
    }

  } else {
    Tcl_SetResult (interp, "expected -time, -blocking or -nonblocking", TCL_STATIC);
    return -1;
  } 

    /* we got the number of seconds */

  return ++i;
} 
