/* Agent Tcl
   Bob Gray
   24 June 1995

   tcpipBasic.cc

   This file implements the TCP/IP library routines.

   Copyright (c) 1995, Robert S. Gray Dartmouth College

   See the file "agent.terms" for information on usage and redistribution
   of this file and for a DISCLAIMER OF ALL WARRANTIES.
*/

#ifndef NO_PRAGMAS
#pragma implementation
#endif "tcpipTcl.h"

#include "platPorting.h"
#include "platSystem.h"
#include "agentMachId.h"     // MachineId class
#include "genTcpip.h"

/* ------------------------------------------------------------------- */

#include "platTimers.h"
#include "platTimeval.h"
#include "genTcpip.h"
#include "genUtility.h"
#include "tcl.h"
#include "tcpipTcl.h"
#include "truefalse.h"

/* Tcpip_Getip

   Purpose: Get the IP number of a machine.

     Input: interp  = the current interpreter
	      	      (struct Tcl_Interp *)

	    seconds = maximum number of seconds to wait for the IP address
		      (double)

	    server  = name of the machine
		      (char *)

    Output: The procedure returns TCL_ERROR and sets the interpreter result 
	    to an appropriate error message on error.  Otherwise the 
	    procedure returns TCL_OK and fills in "ip".
*/

int Tcpip_Getip (Tcl_Interp *interp, double seconds, char *server, UINT_32 *ip)
{
    int rc;
    struct timeval stop;
    MachineId id = MachineId (DynamicString(server), UNKNOWN_IP);

	/* assertions on the parameters */

    assert (interp != NULL);
    assert (seconds >= 0.0);
    assert (server != NULL);
    assert (ip != NULL);

	/* construct the stop time */

    stop = TimevalUtil::addTimevals (SystemUtil::getCurrentWall(), TimevalUtil::secondsToTimeval(seconds));

	/* get the IP address */

    if ((rc = tcpip_getip (id, stop)) == TCPIP_TIMEOUT) {
	Tcl_AppendResult (interp, "timeout while getting IP address of \"", id.getName().value(), "\"", (char *) NULL);
	return TCL_ERROR; 
    } 

    if (rc < 0) {
        Tcl_AppendResult (interp, "unable to get IP address of \"", id.getName().value(), "\"", (char *) NULL);
	return TCL_ERROR;
    }

    *ip = id.getIp();
    return TCL_OK;
}
