/* Agent Tcl
   Bob Gray
   11 July 1995

   genTransmit.h

   This file defines the routines that send and receive messages.

   Copyright (c) 1995, Robert S. Gray Dartmouth College

   See the file "agent.terms" for information on usage and redistribution
   of this file and for a DISCLAIMER OF ALL WARRANTIES.
*/

#ifndef _GEN_TRANSMIT_H
#define _GEN_TRANSMIT_H

#ifndef NO_PRAGMAS
#pragma interface
#endif

#ifndef _AGENT_MACH_ID_H
#include "agentMachId.h"		// MachineId
#endif

#include "agentId.h"
#include "genMessage.h"
#include "genSecurity.h"

    /* forward decalartion so we can compile */

class Connection;

    /* error codes */

const int MESSAGE_OK     	  =  0;
const int MESSAGE_TIMEOUT	  = -1;
const int MESSAGE_SYS_ERROR	  = -2;
const int MESSAGE_INVALID    	  = -3;
const int MESSAGE_ENCRYPT 	  = -4;
const int MESSAGE_DECRYPT	  = -5;
const int MESSAGE_NOT_FOUND	  = -6;
const int MESSAGE_NO_SERVER	  = -7;
const int MESSAGE_CANT_READ	  = -8;
const int MESSAGE_CANT_WRITE	  = -9;
const int MESSAGE_PENDING	  = -10;
const int MESSAGE_LOST_CONNECTION = -11;

    /* round-trip information */

struct RoundTrip
{
    MachineId machine;          /* recipient machine       */
    struct timeval startTime;    /* transmission start time */
    struct timeval endTime;      /* transmission end time   */
    UINT_32 bytesSent;           /* bytes sent              */
    UINT_32 bytesReceived;       /* bytes received          */

	/* constructors */

    RoundTrip (const MachineId& p_machine):
	machine (p_machine),
	bytesSent (0),
	bytesReceived (0)
    {
	startTime.tv_sec  = 0;     // is there a way to this in init. list?
	startTime.tv_usec = 0;
	endTime.tv_sec    = 0;
	endTime.tv_usec   = 0;
    }

    RoundTrip (const RoundTrip& trip):
	machine (trip.machine),
	startTime (trip.startTime),
	endTime (trip.endTime),
	bytesSent (trip.bytesSent),
	bytesReceived (trip.bytesReceived)
    {
	// empty
    }

	/* assignment operator */

    RoundTrip& operator= (const RoundTrip& trip) {

	if (this != &trip) {
	    machine = trip.machine;
	    startTime = trip.startTime;
	    endTime = trip.endTime;
	    bytesSent = trip.bytesSent;
	    bytesReceived = trip.bytesReceived;
	}

	return (*this);
    }
};

    /* receive a message and perform any necessary descryption -- will go away soon */

int secureReceive
	(int sockfd, const MESSAGE_SET &set, const BASE_SECURITY &receiver,
	 BASE_SECURITY &sender, MESSAGE *&message);

    /* basic send and receive -- building blocks for all of the above */

int message_send 
	(int sockfd, MESSAGE &message);

int message_send
	(Connection &connection, MESSAGE &message);

MESSAGE *message_receive 
	(int sockfd, const MESSAGE_SET &set);

MESSAGE *message_receive
	(Connection &connection, const MESSAGE_SET &set);

#endif 
